import json
import os
import uuid
import zipfile
import tkinter as tk
from tkinter import ttk, filedialog, simpledialog, messagebox
import requests # For downloading to get file size
import tempfile # For temporary file/directory for downloads

# --- Constants ---
MANIFEST_FILE_NAME = "store_manifest.json"
APKS_DIR_NAME = "apks"
IMAGES_DIR_NAME = "images"

# --- Helper Class for App Data ---
class AppEntry:
    def __init__(self, apk_path_or_url, is_url=False):
        self.is_url = is_url
        self.apk_source = apk_path_or_url
        
        if is_url:
            try:
                # A very basic way to guess filename from URL
                self.apk_file_name = os.path.basename(requests.utils.urlparse(apk_path_or_url).path)
                if not self.apk_file_name or self.apk_file_name == '/': # If path is just '/' or empty
                    self.apk_file_name = f"app_from_url_{uuid.uuid4().hex[:6]}.apk"
            except Exception: # If URL parsing fails for any reason
                 self.apk_file_name = f"app_from_url_{uuid.uuid4().hex[:6]}.apk"
            self.app_name_var = tk.StringVar(value=os.path.splitext(self.apk_file_name)[0])
        else: # Local file
            self.apk_file_name = os.path.basename(apk_path_or_url)
            self.app_name_var = tk.StringVar(value=os.path.splitext(self.apk_file_name)[0])

        self.package_name_var = tk.StringVar(value=self._generate_package_name())
        self.description_var = tk.StringVar(value=f"תיאור עבור {self.app_name_var.get()}")
        self.image_path_var = tk.StringVar(value="") # Full path to local image file
        self.image_file_name_in_zip_var = tk.StringVar(value="") # Just the filename for the ZIP
        self.file_size_var = tk.StringVar(value="0 B")

    def _generate_package_name(self):
        safe_app_name = ''.join(e for e in self.app_name_var.get().lower() if e.isalnum() or e == '_').replace(" ", "_")
        if not safe_app_name:
            safe_app_name = "unknownapp"
        return f"com.generated.{safe_app_name}.{uuid.uuid4().hex[:6]}"

    def to_dict(self):
        size_str = self.file_size_var.get()
        size_bytes = 0
        try:
            if "KB" in size_str.upper():
                size_bytes = int(float(size_str.upper().replace("KB","").strip()) * 1024)
            elif "MB" in size_str.upper():
                size_bytes = int(float(size_str.upper().replace("MB","").strip()) * 1024 * 1024)
            elif "B" in size_str.upper() and "KB" not in size_str.upper() and "MB" not in size_str.upper() : # ensure it's just B
                size_bytes = int(float(size_str.upper().replace("B","").strip()))
        except ValueError:
            size_bytes = 0 # Default to 0 if conversion fails
        
        return {
            "name": self.app_name_var.get(),
            "description": self.description_var.get(),
            "apkFileName": self.apk_file_name,
            "imageFileName": self.image_file_name_in_zip_var.get() if self.image_file_name_in_zip_var.get() else None,
            "packageName": self.package_name_var.get(),
            "apkUrl": self.apk_source if self.is_url else None,
            "fileSize": size_bytes
        }

    def update_image_selection(self, image_path):
        self.image_path_var.set(image_path)
        self.image_file_name_in_zip_var.set(os.path.basename(image_path) if image_path else "")

# --- Main Application Class (Rest of the code is identical to the previous full GUI script) ---
class BackupCreatorApp:
    def __init__(self, root_window):
        self.root = root_window
        root_window.title("Kosher App Store - Backup Creator")
        root_window.geometry("850x750")

        self.selected_apks_data = []

        paned_window = ttk.PanedWindow(root_window, orient=tk.HORIZONTAL)
        paned_window.pack(fill=tk.BOTH, expand=True, padx=10, pady=10)

        left_pane = ttk.Frame(paned_window, padding="10")
        paned_window.add(left_pane, weight=2)

        settings_frame = ttk.LabelFrame(left_pane, text="Store General Settings", padding="10")
        settings_frame.pack(pady=10, fill="x")
        self._create_settings_widgets(settings_frame)

        apks_list_frame = ttk.LabelFrame(left_pane, text="APKs to Include / Edit", padding="10")
        apks_list_frame.pack(pady=10, fill="both", expand=True)
        self._create_apk_list_widgets(apks_list_frame)

        right_pane = ttk.Frame(paned_window, padding="10")
        paned_window.add(right_pane, weight=3)

        self.selected_apk_details_frame = ttk.LabelFrame(right_pane, text="Edit Selected App Details", padding="10")
        self.selected_apk_details_frame.pack(fill="both", expand=True)
        self._create_apk_details_widgets(self.selected_apk_details_frame)

        action_buttons_frame = ttk.Frame(root_window, padding="10")
        action_buttons_frame.pack(fill="x")
        self._create_action_buttons(action_buttons_frame)

        self.clear_apk_details_fields()

    def _create_settings_widgets(self, parent_frame):
        self.password_var = tk.StringVar(value="yossi6547")
        self.normal_install_var = tk.BooleanVar(value=False)
        self.use_root_var = tk.BooleanVar(value=True)
        self.root_method_var = tk.StringVar(value="pm_install")
        self.install_location_var = tk.StringVar(value="system/app")
        self.send_feedback_var = tk.BooleanVar(value=True)
        
        r = 0
        ttk.Label(parent_frame, text="Store Password:").grid(row=r, column=0, sticky="w", padx=5, pady=2)
        ttk.Entry(parent_frame, textvariable=self.password_var, width=30).grid(row=r, column=1, sticky="ew", padx=5, pady=2); r+=1
        
        ttk.Checkbutton(parent_frame, text="Default: Normal Install Mode", variable=self.normal_install_var).grid(row=r, column=0, columnspan=2, sticky="w", padx=5, pady=2); r+=1
        ttk.Checkbutton(parent_frame, text="Default: Use Root Install", variable=self.use_root_var).grid(row=r, column=0, columnspan=2, sticky="w", padx=5, pady=2); r+=1
        
        ttk.Label(parent_frame, text="Default Root Method:").grid(row=r, column=0, sticky="w", padx=5, pady=2)
        ttk.Radiobutton(parent_frame, text="PM Install", variable=self.root_method_var, value="pm_install").grid(row=r, column=1, sticky="w", padx=5, pady=0); r+=1
        ttk.Radiobutton(parent_frame, text="Copy to System", variable=self.root_method_var, value="copy_to_system").grid(row=r, column=1, sticky="w", padx=5, pady=0); r+=1
        
        ttk.Label(parent_frame, text="Default Copy Location:").grid(row=r, column=0, sticky="w", padx=5, pady=2)
        ttk.Radiobutton(parent_frame, text="/system/app", variable=self.install_location_var, value="system/app").grid(row=r, column=1, sticky="w", padx=5, pady=0); r+=1
        ttk.Radiobutton(parent_frame, text="/system/priv-app", variable=self.install_location_var, value="system/priv-app").grid(row=r, column=1, sticky="w", padx=5, pady=0); r+=1
        
        ttk.Checkbutton(parent_frame, text="Enable Send Feedback in Menu", variable=self.send_feedback_var).grid(row=r, column=0, columnspan=2, sticky="w", padx=5, pady=2); r+=1
        
        ttk.Label(parent_frame, text="About Text:").grid(row=r, column=0, sticky="nw", padx=5, pady=2)
        self.about_text_widget = tk.Text(parent_frame, height=4, width=30, wrap=tk.WORD)
        self.about_text_widget.grid(row=r, column=1, sticky="ew", padx=5, pady=2)
        self.about_text_widget.insert(tk.END, "זוהי חנות אפליקציות כשרה שנוצרה באמצעות סקריפט הגיבוי.")
        parent_frame.columnconfigure(1, weight=1)

    def _create_apk_list_widgets(self, parent_frame):
        self.apk_listbox = tk.Listbox(parent_frame, selectmode=tk.SINGLE, height=10, exportselection=False) # exportselection=False
        self.apk_listbox.pack(side=tk.LEFT, fill="both", expand=True, padx=(0, 5))
        self.apk_listbox.bind("<<ListboxSelect>>", self.on_apk_select_from_list)
        apk_scrollbar = ttk.Scrollbar(parent_frame, orient="vertical", command=self.apk_listbox.yview)
        apk_scrollbar.pack(side=tk.LEFT, fill="y")
        self.apk_listbox.config(yscrollcommand=apk_scrollbar.set)

        buttons_subframe = ttk.Frame(parent_frame)
        buttons_subframe.pack(side=tk.BOTTOM, fill="x", pady=(5,0))
        add_local_apk_button = ttk.Button(buttons_subframe, text="Add Local APK(s)", command=self.add_local_apks_handler)
        add_local_apk_button.pack(side=tk.LEFT, padx=2, pady=2)
        add_url_apk_button = ttk.Button(buttons_subframe, text="Add APK from URL", command=self.add_url_apk_handler)
        add_url_apk_button.pack(side=tk.LEFT, padx=2, pady=2)
        remove_apk_button = ttk.Button(buttons_subframe, text="Remove Selected", command=self.remove_selected_apk_handler)
        remove_apk_button.pack(side=tk.LEFT, padx=2, pady=2)

    def _create_apk_details_widgets(self, parent_frame):
        self.selected_apk_source_label = ttk.Label(parent_frame, text="Source: N/A", wraplength=300)
        self.selected_apk_source_label.pack(anchor="w", pady=2, fill="x")

        ttk.Label(parent_frame, text="App Name (from filename/URL):").pack(anchor="w", pady=(5,0))
        self.selected_apk_appname_entry = ttk.Entry(parent_frame, width=40)
        self.selected_apk_appname_entry.pack(anchor="w", fill="x", pady=2)
        self.selected_apk_appname_entry.bind("<KeyRelease>", self.update_selected_apk_details_from_fields)

        ttk.Label(parent_frame, text="Package Name (auto-generated, editable):").pack(anchor="w", pady=(5,0))
        self.selected_apk_package_entry = ttk.Entry(parent_frame, width=40)
        self.selected_apk_package_entry.pack(anchor="w", fill="x", pady=2)
        self.selected_apk_package_entry.bind("<KeyRelease>", self.update_selected_apk_details_from_fields)

        ttk.Label(parent_frame, text="Description:").pack(anchor="w", pady=(5,0))
        self.selected_apk_description_text = tk.Text(parent_frame, height=5, width=40, wrap=tk.WORD)
        self.selected_apk_description_text.pack(anchor="w", fill="both", pady=2, expand=True)
        self.selected_apk_description_text.bind("<KeyRelease>", self.update_selected_apk_details_from_fields)
        
        image_frame = ttk.Frame(parent_frame)
        image_frame.pack(anchor="w", fill="x", pady=(10,0))
        self.selected_apk_image_label_var = tk.StringVar(value="Image: None")
        ttk.Label(image_frame, textvariable=self.selected_apk_image_label_var).pack(side=tk.LEFT, pady=2, padx=(0,5))
        ttk.Button(image_frame, text="Select Image...", command=self.select_image_for_selected_apk).pack(side=tk.LEFT, pady=2)

        size_frame = ttk.Frame(parent_frame)
        size_frame.pack(anchor="w", fill="x", pady=(10,0))
        self.selected_apk_size_label_var = tk.StringVar(value="Size: N/A")
        ttk.Label(size_frame, textvariable=self.selected_apk_size_label_var).pack(side=tk.LEFT, pady=2, padx=(0,5))
        ttk.Button(size_frame, text="Calc. Size (from URL)", command=self.calculate_apk_size_handler).pack(side=tk.LEFT, pady=2)

    def _create_action_buttons(self, parent_frame):
        create_button = ttk.Button(parent_frame, text="Create Backup ZIP", command=self.create_backup_zip_handler, style="Accent.TButton")
        create_button.pack(pady=10)
        style = ttk.Style()
        style.configure("Accent.TButton", font=("Segoe UI", 10, "bold"))

    def add_local_apks_handler(self):
        # ... (Same as before)
        files = filedialog.askopenfilenames(title="Select Local APK files", filetypes=(("APK files", "*.apk"),))
        if files:
            for f_path in files:
                if not any(entry.apk_source == f_path for entry in self.selected_apks_data):
                    app_entry = AppEntry(f_path, is_url=False)
                    self.selected_apks_data.append(app_entry)
                    self.apk_listbox.insert(tk.END, app_entry.apk_file_name)
            if self.selected_apks_data and self.apk_listbox.size() > 0:
                 self.apk_listbox.selection_clear(0, tk.END)
                 self.apk_listbox.selection_set(self.apk_listbox.size() -1)
                 self.on_apk_select_from_list(None)


    def add_url_apk_handler(self):
        # ... (Same as before)
        url = simpledialog.askstring("Add APK from URL", "Enter APK URL:")
        if url:
            if not any(entry.apk_source == url for entry in self.selected_apks_data):
                app_entry = AppEntry(url, is_url=True)
                self.selected_apks_data.append(app_entry)
                self.apk_listbox.insert(tk.END, app_entry.apk_file_name)
            if self.selected_apks_data and self.apk_listbox.size() > 0:
                self.apk_listbox.selection_clear(0, tk.END)
                self.apk_listbox.selection_set(self.apk_listbox.size() -1)
                self.on_apk_select_from_list(None)

    def remove_selected_apk_handler(self):
        # ... (Same as before)
        selected_indices = self.apk_listbox.curselection()
        if selected_indices:
            index_to_remove = selected_indices[0]
            self.apk_listbox.delete(index_to_remove)
            del self.selected_apks_data[index_to_remove]
            if self.apk_listbox.size() > 0:
                new_selection_index = max(0, index_to_remove -1) if index_to_remove == self.apk_listbox.size() else index_to_remove
                if new_selection_index < self.apk_listbox.size(): self.apk_listbox.selection_set(new_selection_index)
                self.on_apk_select_from_list(None)
            else: self.clear_apk_details_fields()

    def on_apk_select_from_list(self, event):
        # ... (Same as before, with state handling for entries)
        selected_indices = self.apk_listbox.curselection()
        if selected_indices:
            index = selected_indices[0]
            app_entry = self.selected_apks_data[index]
            self.selected_apk_source_label.config(text=f"Source: {app_entry.apk_source}")
            self.selected_apk_appname_entry.config(state=tk.NORMAL)
            self.selected_apk_appname_entry.delete(0, tk.END)
            self.selected_apk_appname_entry.insert(0, app_entry.app_name_var.get())
            self.selected_apk_package_entry.config(state=tk.NORMAL)
            self.selected_apk_package_entry.delete(0, tk.END)
            self.selected_apk_package_entry.insert(0, app_entry.package_name_var.get())
            self.selected_apk_description_text.config(state=tk.NORMAL)
            self.selected_apk_description_text.delete("1.0", tk.END)
            self.selected_apk_description_text.insert("1.0", app_entry.description_var.get())
            self.selected_apk_image_label_var.set(f"Image: {os.path.basename(app_entry.image_path_var.get()) if app_entry.image_path_var.get() else 'None'}")
            self.selected_apk_size_label_var.set(f"Size: {app_entry.file_size_var.get()}")
        else:
            self.clear_apk_details_fields()


    def update_selected_apk_details_from_fields(self, event=None):
        # ... (Same as before)
        selected_indices = self.apk_listbox.curselection()
        if selected_indices:
            index = selected_indices[0]
            self.selected_apks_data[index].app_name_var.set(self.selected_apk_appname_entry.get())
            self.selected_apks_data[index].package_name_var.set(self.selected_apk_package_entry.get())
            self.selected_apks_data[index].description_var.set(self.selected_apk_description_text.get("1.0", tk.END).strip())
            # Update listbox display if app name changed (which is part of display)
            self.apk_listbox.delete(index)
            self.apk_listbox.insert(index, self.selected_apks_data[index].apk_file_name) # Assuming display is still filename
            self.apk_listbox.selection_set(index)


    def select_image_for_selected_apk(self):
        # ... (Same as before)
        selected_indices = self.apk_listbox.curselection()
        if not selected_indices: messagebox.showwarning("Selection Error", "Please select an APK from the list first."); return
        index = selected_indices[0]
        image_path = filedialog.askopenfilename(title=f"Select Image for {self.selected_apks_data[index].app_name_var.get()}", filetypes=(("Image files", "*.png *.jpg *.jpeg *.gif"),))
        if image_path:
            self.selected_apks_data[index].update_image_selection(image_path)
            self.selected_apk_image_label_var.set(f"Image: {os.path.basename(image_path)}")


    def calculate_apk_size_handler(self):
        # ... (Same as before, with tempfile usage)
        selected_indices = self.apk_listbox.curselection()
        if not selected_indices: messagebox.showwarning("Selection Error", "Please select an APK from the list first."); return
        index = selected_indices[0]; app_entry = self.selected_apks_data[index]
        if not app_entry.is_url:
            try: size_bytes = os.path.getsize(app_entry.apk_source); app_entry.file_size_var.set(self.format_size(size_bytes)); self.selected_apk_size_label_var.set(f"Size: {app_entry.file_size_var.get()}")
            except OSError: messagebox.showerror("Error", f"Could not get size for local file: {app_entry.apk_source}"); return
            return
        url = app_entry.apk_source; self.selected_apk_size_label_var.set("Size: Calculating..."); self.root.update_idletasks()
        try:
            response_head = requests.head(url, timeout=5, allow_redirects=True); response_head.raise_for_status()
            content_length = response_head.headers.get('content-length')
            if content_length:
                size_bytes = int(content_length); app_entry.file_size_var.set(self.format_size(size_bytes)); self.selected_apk_size_label_var.set(f"Size: {app_entry.file_size_var.get()}"); messagebox.showinfo("Size Calculated", f"File size (headers): {app_entry.file_size_var.get()}"); return
            
            messagebox.showinfo("Calculating Size", "Attempting temporary download for size. This may take a moment.")
            with tempfile.TemporaryDirectory() as temp_dir: # Use TemporaryDirectory
                temp_apk_path = os.path.join(temp_dir, "temp_download.apk")
                with requests.get(url, stream=True, timeout=60) as r: # Increased timeout
                    r.raise_for_status()
                    with open(temp_apk_path, 'wb') as f:
                        for chunk in r.iter_content(chunk_size=8192*4): f.write(chunk) # Larger chunk
                size_bytes = os.path.getsize(temp_apk_path)
                app_entry.file_size_var.set(self.format_size(size_bytes)); self.selected_apk_size_label_var.set(f"Size: {app_entry.file_size_var.get()}"); messagebox.showinfo("Size Calculated", f"File size (download): {app_entry.file_size_var.get()}")
        except requests.exceptions.RequestException as e: app_entry.file_size_var.set("Error"); self.selected_apk_size_label_var.set("Size: Error"); messagebox.showerror("Network Error", f"Could not fetch file size: {e}")
        except Exception as e: app_entry.file_size_var.set("Error"); self.selected_apk_size_label_var.set("Size: Error"); messagebox.showerror("Error", f"An error occurred: {e}")


    def format_size(self, size_bytes):
        # ... (Same as before)
        if size_bytes < 1024: return f"{size_bytes} B"
        elif size_bytes < 1024*1024: return f"{size_bytes/1024:.2f} KB"
        else: return f"{size_bytes/(1024*1024):.2f} MB"

    def clear_apk_details_fields(self):
        # ... (Same as before, with state handling)
        self.selected_apk_source_label.config(text="Source: N/A")
        self.selected_apk_appname_entry.delete(0, tk.END); self.selected_apk_appname_entry.config(state=tk.DISABLED)
        self.selected_apk_package_entry.delete(0, tk.END); self.selected_apk_package_entry.config(state=tk.DISABLED)
        self.selected_apk_description_text.delete("1.0", tk.END); self.selected_apk_description_text.config(state=tk.DISABLED)
        self.selected_apk_image_label_var.set("Image: None")
        self.selected_apk_size_label_var.set("Size: N/A")


    def create_backup_zip_handler(self):
        # ... (Same as before)
        if not self.selected_apks_data: messagebox.showerror("Error", "No APK files selected."); return
        output_zip_path = filedialog.asksaveasfilename(title="Save Backup ZIP", defaultextension=".zip", filetypes=(("ZIP files", "*.zip"),))
        if not output_zip_path: messagebox.showwarning("Cancelled", "Save location not selected."); return
        app_info_list_for_json = [entry.to_dict() for entry in self.selected_apks_data]
        store_config_data = { "appList": app_info_list_for_json, "settingsPassword": self.password_var.get(), "normalInstallMode": self.normal_install_var.get(), "useRootInstall": self.use_root_var.get(), "rootInstallMethod": self.root_method_var.get(), "installLocation": self.install_location_var.get(), "enableSendFeedback": self.send_feedback_var.get(), "aboutText": self.about_text_widget.get("1.0", tk.END).strip() }
        manifest_json_content = json.dumps(store_config_data, indent=4, ensure_ascii=False)
        try:
            with zipfile.ZipFile(output_zip_path, 'w', zipfile.ZIP_DEFLATED) as zf:
                zf.writestr(MANIFEST_FILE_NAME, manifest_json_content)
                for app_entry in self.selected_apks_data:
                    if not app_entry.is_url: zf.write(app_entry.apk_source, os.path.join(APKS_DIR_NAME, app_entry.apk_file_name))
                    if app_entry.image_path_var.get(): zf.write(app_entry.image_path_var.get(), os.path.join(IMAGES_DIR_NAME, app_entry.image_file_name_in_zip_var.get()))
            messagebox.showinfo("Success", f"Backup created successfully at:\n{output_zip_path}")
        except Exception as e: messagebox.showerror("Error", f"Failed to create backup: {e}"); print(f"Error: {e}")


if __name__ == "__main__":
    root = tk.Tk()
    app = BackupCreatorApp(root)
    root.mainloop()